function init() {
	var atom_common_table_header = struct({
		structuresize: uint16().setValidation(function () {
			return this.value == 2470 ? true : "The pp_table for sienna_cichlid is supposed to be 2470 bytes";
		}),
		format_revision: uint8(),  //mainly used for a hw function, when the parser is not backward compatible 
		content_revision: uint8(),  //change it when a data table has a structure change, or a hw function has a input/output parameter change
	}).setValidation(function () {
		if (this.format_revision.value != 15 || this.content_revision.value != 0)
			return "//For sienna_cichlid, header.format_revision = 15, header.content_revision = 0";
		return true;
	});

	var QuadraticInt = struct({
		a: float(),  // store in IEEE float format in this variable
		b: float(),  // store in IEEE float format in this variable
		c: float(),  // store in IEEE float format in this variable
	});

	var LinearInt = struct({            
		m: float(),  // store in IEEE float format in this variable
		b: float(),  // store in IEEE float format in this variable
	});

	var DroopInt = struct({            
		a: float(),  // store in IEEE float format in this variable
		b: float(),  // store in IEEE float format in this variable
		c: float(),  // store in IEEE float format in this variable
	});

	const NUM_PIECE_WISE_LINEAR_DROOP_MODEL_VF_POINTS = 5;
	var PiecewiseLinearDroopInt = struct({
		Fset: array(float(), NUM_PIECE_WISE_LINEAR_DROOP_MODEL_VF_POINTS),  //in GHz, store in IEEE float format
		Vdroop: array(float(), NUM_PIECE_WISE_LINEAR_DROOP_MODEL_VF_POINTS),  //in V  , store in IEEE float format
	});

	var DpmDescriptor = struct({
		VoltageMode: enumeration("Voltage Mode", "uint8", {
			"AVFS only": 0,
			"min(AVFS,SS)": 1,
			"SS only": 2,
		}),  // 0 - AVFS only, 1- min(AVFS,SS), 2-SS only
		SnapToDiscrete: bool8(),  // 0 - Fine grained DPM, 1 - Discrete DPM
		NumDiscreteLevels: uint8(),  // Set to 2 (Fmin, Fmax) when using fine grained DPM, otherwise set to # discrete levels used
		Padding: uint8(),         
		ConversionToAvfsClk: LinearInt,  // Transfer function to AVFS Clock (GHz->GHz)
		SsCurve: QuadraticInt,  // Slow-slow curve (GHz->V)
		SsFmin: uint16(),  // Fmin for SS curve. If SS curve is selected, will use V@SSFmin for F <= Fmin
		Padding16: uint16(),    
	});

	// Used for 2-step UCLK DPM change workaround
	var UclkDpmChangeRange = struct({
		Fmin: uint16(),
		Fmax: uint16(),
	});

	var I2cControllerConfig = struct({
		Enabled: uint8(),
		Speed: uint8(),
		SlaveAddress: uint8(),  
		ControllerPort: uint8(),
		ControllerName: uint8(),
		ThermalThrotter: uint8(),
		I2cProtocol: uint8(),
		PaddingConfig: uint8(),  
	});

	const PPT_THROTTLER_COUNT = 4;

	var TdcThrottlers = struct({
		Gfx: uint16(),
		Soc: uint16(),
	});

	var Temps = struct({
		Edge: uint16(),
		Hotspot: uint16(),
		Mem: uint16(),
		VrGfx: uint16(),
		VrMem0: uint16(),
		VrMem1: uint16(),
		VrSoc: uint16(),
		Liquid0: uint16(),
		Liquid1: uint16(),
		Plx: uint16(),
	});

	const NUM_GFXCLK_DPM_LEVELS = 16;
	const NUM_SMNCLK_DPM_LEVELS = 2;
	const NUM_SOCCLK_DPM_LEVELS = 8;
	const NUM_MP0CLK_DPM_LEVELS = 2;
	const NUM_DCLK_DPM_LEVELS = 8;
	const NUM_VCLK_DPM_LEVELS = 8;
	const NUM_DCEFCLK_DPM_LEVELS = 8;
	const NUM_PHYCLK_DPM_LEVELS = 8;
	const NUM_DISPCLK_DPM_LEVELS = 8;
	const NUM_PIXCLK_DPM_LEVELS = 8;
	const NUM_DTBCLK_DPM_LEVELS = 8;
	const NUM_UCLK_DPM_LEVELS = 4; 
	const NUM_MP1CLK_DPM_LEVELS = 2;
	const NUM_LINK_LEVELS = 2;
	const NUM_FCLK_DPM_LEVELS = 8; 
	const NUM_XGMI_LEVELS = 2;
	const NUM_XGMI_PSTATE_LEVELS = 4;
	const NUM_OD_FAN_MAX_POINTS = 6;

	//RLC Pace Table total number of levels
	const RLC_PACE_TABLE_NUM_LEVELS = 16;
	const SIENNA_CICHLID_UMC_CHANNEL_NUM = 16;

	const AVFS_VOLTAGE_COUNT = 2;
	// TODO: AVFS Voltage structs

	const NUM_I2C_CONTROLLERS = 16;

	var PPTable = struct({
		// MAJOR SECTION
		"SKU Parameters": struct({

			Version: uint32(),

			// SECTION:
			"Feature Enablement": struct({
				FeaturesToRun: flags("feature bits", "uint64", {
					// Feature Control Defines
					// DPM
					"DPM_PREFETCHER": 1 << 0,
					"DPM_GFXCLK": 1 << 1,
					"DPM_GFX_GPO": 1 << 2,
					"DPM_UCLK": 1 << 3,
					"DPM_FCLK": 1 << 4,
					"DPM_SOCCLK": 1 << 5,
					"DPM_MP0CLK": 1 << 6,
					"DPM_LINK": 1 << 7,
					"DPM_DCEFCLK": 1 << 8,
					"DPM_XGMI": 1 << 9,
					"MEM_VDDCI_SCALING": 1 << 10,
					"MEM_MVDD_SCALING": 1 << 11,

					//Idle
					"DS_GFXCLK": 1 << 12,
					"DS_SOCCLK": 1 << 13,
					"DS_FCLK": 1 << 14,
					"DS_LCLK": 1 << 15,
					"DS_DCEFCLK": 1 << 16,
					"DS_UCLK": 1 << 17,
					"GFX_ULV": 1 << 18,
					"FW_DSTATE": 1 << 19,
					"GFXOFF": 1 << 20,
					"BACO": 1 << 21,
					"MM_DPM_PG": 1 << 22,
					//Throttler/Response
					"PPT": 1 << 24,
					"TDC": 1 << 25,
					"APCC_PLUS": 1 << 26,
					"GTHR": 1 << 27,
					"ACDC": 1 << 28,
					"VR0HOT": 1 << 29,
					"VR1HOT": 1 << 30,
					"FW_CTF": 1 << 31,
					"FAN_CONTROL": 1 << 32,
					"THERMAL": 1 << 33,
					"GFX_DCS": 1 << 34,
					//VF
					"RM": 1 << 35,
					"LED_DISPLAY": 1 << 36,
					//Other
					"GFX_SS": 1 << 37,
					"OUT_OF_BAND_MONITOR": 1 << 38,
					"TEMP_DEPENDENT_VMIN": 1 << 39,

					"MMHUB_PG": 1 << 40,
					"ATHUB_PG": 1 << 41,
					"APCC_DFLL": 1 << 42,
					"DF_SUPERV": 1 << 43,
					"RSMU_SMN_CG": 1 << 44,
					"DF_CSTATE": 1 << 45,
					"2_STEP_PSTATE": 1 << 46,
					"SMNCLK_DPM": 1 << 47,
					"PERLINK_GMIDOWN": 1 << 48,
					"GFX_EDC": 1 << 49,
					"GFX_PER_PART_VMIN": 1 << 50,
					"SMART_SHIFT": 1 << 51,
					"APT": 1 << 52,
				}),

			}),
			// SECTION:
			"Infrastructure Limits": struct({
				SocketPowerLimitAc: array(uint16(), PPT_THROTTLER_COUNT),  // Watts
				SocketPowerLimitAcTau: array(uint16(), PPT_THROTTLER_COUNT),  // Time constant of LPF in ms
				SocketPowerLimitDc: array(uint16(), PPT_THROTTLER_COUNT),  // Watts
				SocketPowerLimitDcTau: array(uint16(), PPT_THROTTLER_COUNT),  // Time constant of LPF in ms

				TdcLimit: TdcThrottlers,  // Amps
				TdcLimitTau: TdcThrottlers,  // Time constant of LPF in ms

				TemperatureLimit: Temps,  // Celcius
	
				FitLimit: uint32(),  // Failures in time (failures per million parts over the defined lifetime)

			}),
			// SECTION:
			"Power Configuration": struct({
				TotalPowerConfig: enumeration("Total Power Config", "uint8", {
					"TDP": 0,
					"TGP": 1,
					"TCP Estimated": 2,
					"TCP Measured": 3
				}),  //0-TDP, 1-TGP, 2-TCP Estimated, 3-TCP Measured. Use defines from PwrConfig_e
				TotalPowerPadding: array(uint8(), 3),  

			}),
			// SECTION:
			"APCC Settings": struct({
				ApccPlusResidencyLimit: uint32(),

			}),
			// SECTION:
			"SMNCLK DPM": struct({
				SmnclkDpmFreq: array(uint16(), NUM_SMNCLK_DPM_LEVELS),  // in MHz
				SmnclkDpmVoltage: array(uint16(), NUM_SMNCLK_DPM_LEVELS),  // mV(Q2)

				PaddingAPCC: uint32(),
				PerPartDroopVsetGfxDfll: array(uint16(), NUM_PIECE_WISE_LINEAR_DROOP_MODEL_VF_POINTS),  //In mV(Q2)
				PaddingPerPartDroop: uint16(),

			}),
			// SECTION:
			"Throttler settings": struct({
				ThrottlerControlMask: uint32(),  // See Throtter masks defines

			}),
			// SECTION:
			"FW DSTATE Settings  ": struct({
				FwDStateMask: uint32(),  // See FW DState masks defines

			}),
			// SECTION:
			"ULV Settings": struct({
				UlvVoltageOffsetSoc: uint16(),  // In mV(Q2)
				UlvVoltageOffsetGfx: uint16(),  // In mV(Q2)

				MinVoltageUlvGfx: uint16(),  // In mV(Q2)  Minimum Voltage ("Vmin") of VDD_GFX in ULV mode 
				MinVoltageUlvSoc: uint16(),  // In mV(Q2)  Minimum Voltage ("Vmin") of VDD_SOC in ULV mode

				SocLIVmin: uint16(),  // In mV(Q2)  Long Idle Vmin (deep ULV), for VDD_SOC
				PaddingLIVmin: uint16(),

				GceaLinkMgrIdleThreshold: uint8(),  //Set by SMU FW during enablment of GFXOFF. Controls delay for GFX SDP port disconnection during idle events
				paddingRlcUlvParams: array(uint8(), 3),

			}),
			// SECTION:
			"Voltage Control Parameters": struct({
				MinVoltageGfx: uint16(),  // In mV(Q2) Minimum Voltage ("Vmin") of VDD_GFX
				MinVoltageSoc: uint16(),  // In mV(Q2) Minimum Voltage ("Vmin") of VDD_SOC
				MaxVoltageGfx: uint16(),  // In mV(Q2) Maximum Voltage allowable of VDD_GFX
				MaxVoltageSoc: uint16(),  // In mV(Q2) Maximum Voltage allowable of VDD_SOC

				LoadLineResistanceGfx: uint16(),  // In mOhms with 8 fractional bits
				LoadLineResistanceSoc: uint16(),  // In mOhms with 8 fractional bits

			}),
			// SECTION:
			"Temperature Dependent Vmin": struct({
				VDDGFX_TVmin: uint16(),  //Celcius
				VDDSOC_TVmin: uint16(),  //Celcius
				VDDGFX_Vmin_HiTemp: uint16(),  // mV Q2
				VDDGFX_Vmin_LoTemp: uint16(),  // mV Q2
				VDDSOC_Vmin_HiTemp: uint16(),  // mV Q2
				VDDSOC_Vmin_LoTemp: uint16(),  // mV Q2
	
				VDDGFX_TVminHystersis: uint16(),  // Celcius
				VDDSOC_TVminHystersis: uint16(),  // Celcius

			}),
			// SECTION:
			"DPM Config 1": struct({
				DpmDescriptor: struct({
					Gfxclk: DpmDescriptor,
					Socclk: DpmDescriptor,
					Uclk: DpmDescriptor,
					Fclk: DpmDescriptor,  
					Dclk_0: DpmDescriptor,
					Vclk_0: DpmDescriptor,
					Dclk_1: DpmDescriptor,
					Vclk_1: DpmDescriptor,
					Dcefclk: DpmDescriptor,
					Dispclk: DpmDescriptor,
					Pixclk: DpmDescriptor,
					Phyclk: DpmDescriptor,
					Dtbclk: DpmDescriptor,
				}),

				FreqTableGfx: array(uint16(), NUM_GFXCLK_DPM_LEVELS),  // In MHz
				FreqTableVclk: array(uint16(), NUM_VCLK_DPM_LEVELS),  // In MHz
				FreqTableDclk: array(uint16(), NUM_DCLK_DPM_LEVELS),  // In MHz
				FreqTableSocclk: array(uint16(), NUM_SOCCLK_DPM_LEVELS),  // In MHz
				FreqTableUclk: array(uint16(), NUM_UCLK_DPM_LEVELS),  // In MHz
				FreqTableDcefclk: array(uint16(), NUM_DCEFCLK_DPM_LEVELS),  // In MHz
				FreqTableDispclk: array(uint16(), NUM_DISPCLK_DPM_LEVELS),  // In MHz
				FreqTablePixclk: array(uint16(), NUM_PIXCLK_DPM_LEVELS),  // In MHz
				FreqTablePhyclk: array(uint16(), NUM_PHYCLK_DPM_LEVELS),  // In MHz
				FreqTableDtbclk: array(uint16(), NUM_DTBCLK_DPM_LEVELS),  // In MHz
				FreqTableFclk: array(uint16(), NUM_FCLK_DPM_LEVELS),  // In MHz
				Paddingclks: uint32(),

				PerPartDroopModelGfxDfll: array(DroopInt, NUM_PIECE_WISE_LINEAR_DROOP_MODEL_VF_POINTS),  //GHz ->Vstore in IEEE float format

				DcModeMaxFreq: struct({
					Gfxclk: uint32(),
					Socclk: uint32(),
					Uclk: uint32(),
					Fclk: uint32(),  
					Dclk_0: uint32(),
					Vclk_0: uint32(),
					Dclk_1: uint32(),
					Vclk_1: uint32(),
					Dcefclk: uint32(),
					Dispclk: uint32(),
					Pixclk: uint32(),
					Phyclk: uint32(),
					Dtbclk: uint32(),
				}),  // In MHz
	
				FreqTableUclkDiv: array(uint8(), NUM_UCLK_DPM_LEVELS),  // 0:Div-1, 1:Div-1/2, 2:Div-1/4, NUM_UCLK_DPM_LEVELS    ), 3:Div-1/8
	
				// Used for MALL performance boost
				FclkBoostFreq: uint16(),  // In Mhz
				FclkParamPadding: uint16(),

			}),
			// SECTION:
			"DPM Config 2": struct({
				Mp0clkFreq: array(uint16(), NUM_MP0CLK_DPM_LEVELS),  // in MHz
				Mp0DpmVoltage: array(uint16(), NUM_MP0CLK_DPM_LEVELS),  // mV(Q2)
				MemVddciVoltage: array(uint16(), NUM_UCLK_DPM_LEVELS),  // mV(Q2)
				MemMvddVoltage: array(uint16(), NUM_UCLK_DPM_LEVELS),  // mV(Q2)
				// GFXCLK DPM
				GfxclkFgfxoffEntry: uint16(),  // in Mhz
				GfxclkFinit: uint16(),  // in Mhz 
				GfxclkFidle: uint16(),  // in MHz 
				GfxclkSource: enumeration("Gfxclk Source", "uint8", {
					"PLL": 0,
					"DFLL": 1,
				}),  // 0 = PLL, 1 = DFLL
				GfxclkPadding: uint8(),

				// GFX GPO 
				GfxGpoSubFeatureMask: uint8(),  // bit 0 = PACE, bit 1 = DEM
				GfxGpoEnabledWorkPolicyMask: uint8(),  //Any policy that GPO can be enabled
				GfxGpoDisabledWorkPolicyMask: uint8(),  //Any policy that GPO can be disabled
				GfxGpoPadding: array(uint8(), 1),
				GfxGpoVotingAllow: uint32(),  //For indicating which feature changes should result in a GPO table recalculation

				GfxGpoPadding32: array(uint32(), 4),  

				GfxDcsFopt: uint16(),  // Optimal GFXCLK for DCS in Mhz
				GfxDcsFclkFopt: uint16(),  // Optimal FCLK for DCS in Mhz
				GfxDcsUclkFopt: uint16(),  // Optimal UCLK for DCS in Mhz
	
				DcsGfxOffVoltage: uint16(),  //Voltage in mV(Q2) applied to VDDGFX when entering DCS GFXOFF phase  

				DcsMinGfxOffTime: uint16(),  //Minimum amount of time PMFW shuts GFX OFF as part of GFX DCS phase
				DcsMaxGfxOffTime: uint16(),  //Maximum amount of time PMFW can shut GFX OFF as part of GFX DCS phase at a stretch.

				DcsMinCreditAccum: uint32(),  //Min amount of positive credit accumulation before waking GFX up as part of DCS.

				DcsExitHysteresis: uint16(),  //The min amount of time power credit accumulator should have a value > 0 before SMU exits the DCS throttling phase. 
				DcsTimeout: uint16(),  //This is the amount of time SMU FW waits for RLC to put GFX into GFXOFF before reverting to the fallback mechanism of throttling GFXCLK to Fmin.

				DcsParamPadding: array(uint32(), 5),

				FlopsPerByteTable: array(uint16(), RLC_PACE_TABLE_NUM_LEVELS),  // Q8.8

				// UCLK section
				LowestUclkReservedForUlv: bool8(),  // Set this to 1 if UCLK DPM0 is reserved for ULV-mode only
				PaddingMem: array(uint8(), 3),
	
				UclkDpmPstates: array(uint8(), NUM_UCLK_DPM_LEVELS),  // 4 DPM states, 0-P0, 1-P1, 2-P2, NUM_UCLK_DPM_LEVELS), 3-P3.
	
				// Used for 2-Step UCLK change workaround
				UclkDpmSrcFreqRange: UclkDpmChangeRange,  // In Mhz
				UclkDpmTargFreqRange: UclkDpmChangeRange,  // In Mhz
				UclkDpmMidstepFreq: uint16(),  // In Mhz
				UclkMidstepPadding: uint16(),

				// Link DPM Settings
				PcieGenSpeed: array(uint8(), NUM_LINK_LEVELS),  ///< 0:PciE-gen1 1:PciE-gen2 2:PciE-gen3 3:PciE-gen4
				PcieLaneCount: array(uint8(), NUM_LINK_LEVELS),  ///< 1=x1, 2=x2, 3=x4, 4=x8, 5=x12, NUM_LINK_LEVELS), 6=x16
				LclkFreq: array(uint16(), NUM_LINK_LEVELS),              
 
			}),
			// SECTION:
			"Fan Control": struct({
				FanStopTemp: uint16(),  //Celcius
				FanStartTemp: uint16(),  //Celcius

				FanGain: Temps,

				FanPwmMin: uint16(),
				FanAcousticLimitRpm: uint16(),
				FanThrottlingRpm: uint16(),
				FanMaximumRpm: uint16(),
				MGpuFanBoostLimitRpm: uint16(),  
				FanTargetTemperature: uint16(),
				FanTargetGfxclk: uint16(),
				FanPadding16: uint16(),
				FanTempInputSelect: uint8(),
				FanPadding: uint8(),
				FanZeroRpmEnable: uint8(), 
				FanTachEdgePerRev: uint8(),
		
				// The following are AFC override parameters. Leave at 0 to use FW defaults.
				FuzzyFan_ErrorSetDelta: int16(),
				FuzzyFan_ErrorRateSetDelta: int16(),
				FuzzyFan_PwmSetDelta: int16(),
				FuzzyFan_Reserved: uint16(),

			}),
			// SECTION:
			"AVFS ": struct({
				// Overrides
				OverrideAvfsGb: array(uint8(), AVFS_VOLTAGE_COUNT),
				dBtcGbGfxDfllModelSelect: uint8(),  //0 -> fused piece-wise model, 1 -> piece-wise linear(PPTable), 2 -> quadratic model(PPTable)  
				Padding8_Avfs: uint8(),

				qAvfsGb: array(QuadraticInt, AVFS_VOLTAGE_COUNT),  // GHz->V Override of fused curve 
				dBtcGbGfxPll: DroopInt,  // GHz->V BtcGb
				dBtcGbGfxDfll: DroopInt,  // GHz->V BtcGb
				dBtcGbSoc: DroopInt,  // GHz->V BtcGb
				qAgingGb: array(LinearInt, AVFS_VOLTAGE_COUNT),  // GHz->V 

				PiecewiseLinearDroopIntGfxDfll: PiecewiseLinearDroopInt,  //GHz ->Vstore in IEEE float format

				qStaticVoltageOffset: array(QuadraticInt, AVFS_VOLTAGE_COUNT),  // GHz->V 

				DcTol: array(uint16(), AVFS_VOLTAGE_COUNT),  // mV Q2

				DcBtcEnabled: array(uint8(), AVFS_VOLTAGE_COUNT),
				Padding8_GfxBtc: array(uint8(), 2),

				DcBtcMin: array(uint16(), AVFS_VOLTAGE_COUNT),  // mV Q2
				DcBtcMax: array(uint16(), AVFS_VOLTAGE_COUNT),  // mV Q2

				DcBtcGb: array(uint16(), AVFS_VOLTAGE_COUNT),  // mV Q2
	
			}),
			// SECTION:
			"XGMI": struct({
				XgmiDpmPstates: array(uint8(), NUM_XGMI_LEVELS),  // 2 DPM states, high and low.  0-P0, 1-P1, 2-P2, NUM_XGMI_LEVELS), 3-P3.
				XgmiDpmSpare: array(uint8(), 2),

			}),
			// SECTION:
			"Advanced Options": struct({
				DebugOverrides: uint32(),
				ReservedEquation0: QuadraticInt, 
				ReservedEquation1: QuadraticInt, 
				ReservedEquation2: QuadraticInt, 
				ReservedEquation3: QuadraticInt, 

			}),
			// SECTION:
			"Sku Reserved": struct({
				CustomerVariant: uint8(),

				//VC BTC parameters are only applicable to VDD_GFX domain
				VcBtcEnabled: uint8(),
				VcBtcVminT0: uint16(),  // T0_VMIN
				VcBtcFixedVminAgingOffset: uint16(),  // FIXED_VMIN_AGING_OFFSET 
				VcBtcVmin2PsmDegrationGb: uint16(),  // VMIN_TO_PSM_DEGRADATION_GB 
				VcBtcPsmA: uint32(),  // A_PSM
				VcBtcPsmB: uint32(),  // B_PSM
				VcBtcVminA: uint32(),  // A_VMIN
				VcBtcVminB: uint32(),  // B_VMIN  
	
				//GPIO Board feature
				LedGpio: uint16(),  //GeneriA GPIO flag used to control the radeon LEDs
				GfxPowerStagesGpio: uint16(),  //Genlk_vsync GPIO flag used to control gfx power stages 
	
				SkuReserved: array(uint32(), 8),

			}),
		}),
		// MAJOR SECTION:
		"Board Parameters": struct({
	
			// SECTION:
			"Gaming Clocks": struct({
				GamingClk: array(uint32(), 6),
	
			}),
			// SECTION:
			"I2C Control": struct({
				I2cControllers: array(I2cControllerConfig, NUM_I2C_CONTROLLERS),     
	
				GpioScl: uint8(),  // GPIO Number for SCL Line, used only for CKSVII2C1
				GpioSda: uint8(),  // GPIO Number for SDA Line, used only for CKSVII2C1
				FchUsbPdSlaveAddr: uint8(),  //For requesting USB PD controller S-states via FCH I2C when entering PME turn off
				I2cSpare: array(uint8(), 1),
	
			}),
			// SECTION:
			"SVI2 Board Parameters": struct({
				VddGfxVrMapping: uint8(),  // Use VR_MAPPING* bitfields
				VddSocVrMapping: uint8(),  // Use VR_MAPPING* bitfields
				VddMem0VrMapping: uint8(),  // Use VR_MAPPING* bitfields
				VddMem1VrMapping: uint8(),  // Use VR_MAPPING* bitfields
	
				GfxUlvPhaseSheddingMask: uint8(),  // set this to 1 to set PSI0/1 to 1 in ULV mode
				SocUlvPhaseSheddingMask: uint8(),  // set this to 1 to set PSI0/1 to 1 in ULV mode
				VddciUlvPhaseSheddingMask: uint8(),  // set this to 1 to set PSI0/1 to 1 in ULV mode
				MvddUlvPhaseSheddingMask: uint8(),  // set this to 1 to set PSI0/1 to 1 in ULV mode
	
			}),
			// SECTION:
			"Telemetry Settings": struct({
				GfxMaxCurrent: uint16(),  // in Amps
				GfxOffset: int8(),  // in Amps
				Padding_TelemetryGfx: uint8(),
	
				SocMaxCurrent: uint16(),  // in Amps
				SocOffset: int8(),  // in Amps
				Padding_TelemetrySoc: uint8(),
	
				Mem0MaxCurrent: uint16(),  // in Amps
				Mem0Offset: int8(),  // in Amps
				Padding_TelemetryMem0: uint8(),
	
				Mem1MaxCurrent: uint16(),  // in Amps
				Mem1Offset: int8(),  // in Amps
				Padding_TelemetryMem1: uint8(),
	
				MvddRatio: uint32(),  // This is used for MVDD  Svi2 Div Ratio workaround. It has 16 fractional bits (Q16.16)
	
			}),
			// SECTION:
			"GPIO Settings": struct({
				AcDcGpio: uint8(),  // GPIO pin configured for AC/DC switching
				AcDcPolarity: uint8(),  // GPIO polarity for AC/DC switching
				VR0HotGpio: uint8(),  // GPIO pin configured for VR0 HOT event
				VR0HotPolarity: uint8(),  // GPIO polarity for VR0 HOT event
	
				VR1HotGpio: uint8(),  // GPIO pin configured for VR1 HOT event 
				VR1HotPolarity: uint8(),  // GPIO polarity for VR1 HOT event 
				GthrGpio: uint8(),  // GPIO pin configured for GTHR Event
				GthrPolarity: uint8(),  // replace GPIO polarity for GTHR
	
				// LED Display Settings
				LedPin0: uint8(),  // GPIO number for LedPin[0]
				LedPin1: uint8(),  // GPIO number for LedPin[1]
				LedPin2: uint8(),  // GPIO number for LedPin[2]
				LedEnableMask: uint8(),
	
				LedPcie: uint8(),  // GPIO number for PCIE results
				LedError: uint8(),  // GPIO number for Error Cases
				LedSpare1: array(uint8(), 2),
	
			}),
			// SECTION:
			"Clock Spread Spectrum": struct({
	
				// GFXCLK PLL Spread Spectrum
				PllGfxclkSpreadEnabled: uint8(),  // on or off
				PllGfxclkSpreadPercent: uint8(),  // Q4.4
				PllGfxclkSpreadFreq: uint16(),  // kHz
	
				// GFXCLK DFLL Spread Spectrum
				DfllGfxclkSpreadEnabled: uint8(),  // on or off
				DfllGfxclkSpreadPercent: uint8(),  // Q4.4
				DfllGfxclkSpreadFreq: uint16(),  // kHz
	
				// UCLK Spread Spectrum
				UclkSpreadPadding: uint16(),
				UclkSpreadFreq: uint16(),  // kHz
	
				// FCLK Spread Spectrum
				FclkSpreadEnabled: uint8(),  // on or off
				FclkSpreadPercent: uint8(),  // Q4.4
				FclkSpreadFreq: uint16(),  // kHz
	
			}),
			// SECTION:
			"Memory Config": struct({
				MemoryChannelEnabled: uint32(),  // For DRAM use only, Max 32 channels enabled bit mask. 
	
				DramBitWidth: uint8(),  // For DRAM use only.  See Dram Bit width type defines
				PaddingMem1: array(uint8(), 3),
	
			}),
			// SECTION:
			"Total Board Power": struct({
				TotalBoardPower: uint16(),  //Only needed for TCP Estimated case, where TCP = TGP+Total Board Power
				BoardPowerPadding: uint16(), 
	
			}),
			// SECTION:
			"XGMI Training": struct({
				XgmiLinkSpeed: array(uint8(), NUM_XGMI_PSTATE_LEVELS),
				XgmiLinkWidth: array(uint8(), NUM_XGMI_PSTATE_LEVELS),
	
				XgmiFclkFreq: array(uint16(), NUM_XGMI_PSTATE_LEVELS),
				XgmiSocVoltage: array(uint16(), NUM_XGMI_PSTATE_LEVELS),
	
			}),
			// SECTION:
			"UMC feature flags": struct({
				HsrEnabled: uint8(),
				VddqOffEnabled: uint8(),
				PaddingUmcFlags: array(uint8(), 2),
	
				// UCLK Spread Spectrum
				UclkSpreadPercent: array(uint8(), 16),   
	
			}),
			// SECTION:
			"Board Reserved": struct({
				BoardReserved: array(uint32(), 11),
			}),
		}),
		// Padding for MMHUB - do not modify this
		MmHubPadding: array(uint32(), 8),  // SMU internal use
	});

	const SMU_11_0_7_PPCLOCK_COUNT = 13;
	const SMU_11_0_7_MAX_PPCLOCK = 16;
	var smu_11_0_7_ppclocks = struct({
		Gfxclk: uint32(),
		Socclk: uint32(),
		Uclk: uint32(),
		Fclk: uint32(),
		Dclk0: uint32(),
		Vclk0: uint32(),
		Dclk1: uint32(),
		Vclk1: uint32(),
		Dcefclk: uint32(),
		Dispclk: uint32(),
		Pixclk: uint32(),
		Phyclk: uint32(),
		Dtbclk: uint32(),
		Unknown: array(uint32(), 0),
		Unused: array(uint32(), SMU_11_0_7_MAX_PPCLOCK - SMU_11_0_7_PPCLOCK_COUNT),
	}).setUpdate(function (root) {
		const count = Math.max(this.parent.count.value, SMU_11_0_7_PPCLOCK_COUNT);
		this.Unknown.length = count - SMU_11_0_7_PPCLOCK_COUNT;
		this.Unused.length = SMU_11_0_7_MAX_PPCLOCK - count;
	});

	const SMU_11_0_7_ODSETTING_COUNT = 30;
	const SMU_11_0_7_MAX_ODSETTING = 64;
	var smu_11_0_7_odsettings = struct({
		GfxclkFmax: uint32(),  // MHz
		GfxclkFmin: uint32(),  // MHz
		CustomGfxVfCurve: QuadraticInt,  // a: mV/MHz^2, b: mv/MHz, c: mV
		CustomCurveVftFmin: uint32(),  // MHz
		UclkFmin: uint32(),  // MHz
		UclkFmax: uint32(),  // MHz
		PowerPercentage: uint32(),  // %
		FanRpmMin: uint32(),
		FanRpmAcousticLimit: uint32(),
		FanTargetTemperature: uint32(),  // Degree Celsius
		OperatingTempMax: uint32(),  // Degree Celsius
		AcTiming: uint32(),
		FanZeroRpmControl: uint32(),
		AutoUvEngine: uint32(),
		AutoOcEngine: uint32(),
		AutoOcMemory: uint32(),
		FanCurveTemperature1: uint32(),
		FanCurveSpeed1: uint32(),
		FanCurveTemperature2: uint32(),
		FanCurveSpeed2: uint32(),
		FanCurveTemperature3: uint32(),
		FanCurveSpeed3: uint32(),
		FanCurveTemperature4: uint32(),
		FanCurveSpeed4: uint32(),
		FanCurveTemperature5: uint32(),
		FanCurveSpeed5: uint32(),
		AutoFanAcousticLimit: uint32(),
		PowerMode: uint32(),
		Unknown: array(uint32(), 0),
		Unused: array(uint32(), SMU_11_0_7_MAX_ODSETTING - SMU_11_0_7_ODSETTING_COUNT),
	}).setUpdate(function (root) {
		const count = Math.max(this.parent.setting_count.value, SMU_11_0_7_ODSETTING_COUNT);
		this.Unknown.length = count - SMU_11_0_7_ODSETTING_COUNT;
		this.Unused.length = SMU_11_0_7_MAX_ODSETTING - count;
	});

	const SMU_11_0_7_ODFEATURE_COUNT = 16;
	const SMU_11_0_7_MAX_ODFEATURE = 32;
	const SMU_11_0_7_MAX_PMSETTING = 32;
	var smu_11_0_7_powerplay_table = struct({
		header: atom_common_table_header,  //For sienna_cichlid, header.format_revision = 15, header.content_revision = 0
		table_revision: uint8().setValidation(function () {
			return this.value == 2 ? true : "//For sienna_cichlid, table_revision = 2";
		}),  //For sienna_cichlid, table_revision = 2
		table_size: uint16(),  //Driver portion table size. The offset to smc_pptable including header size
		golden_pp_id: uint32(),  //PPGen use only: PP Table ID on the Golden Data Base
		golden_revision: uint32(),  //PPGen use only: PP Table Revision on the Golden Data Base
		format_id: uint16().setValidation(function () {
			return this.value == 0x80 ? true : "//For sienna_cichlid this should be 0x80";
		}),  //PPGen use only: PPTable for different ASICs. For sienna_cichlid this should be 0x80
		platform_caps: flags("PlatformCaps", "uint32", {
			"POWERPLAY": 0x1,  // This cap indicates whether CCC need to show Powerplay page.
			"SBIOSPOWERSOURCE": 0x2,  // This cap indicates whether power source notificaiton is done by SBIOS instead of OS.
			"HARDWAREDC": 0x4,  // This cap indicates whether DC mode notificaiton is done by GPIO pin directly.
			"BACO": 0x8,  // This cap indicates whether board supports the BACO circuitry.
			"MACO": 0x10,  // This cap indicates whether board supports the MACO circuitry.
			"SHADOWPSTATE": 0x20,  // This cap indicates whether board supports the Shadow Pstate.
		}),  //POWERPLAYABLE::ulPlatformCaps
		                                            
		thermal_controller_type: enumeration("ThermalController", "uint8", {
			"NONE": 0,
			"SIENNA_CICHLID": 28,
		}),  //one of SMU_11_0_7_PP_THERMALCONTROLLER

		small_power_limit1: uint16(),
		small_power_limit2: uint16(),
		boost_power_limit: uint16(),  //For Gemini Board, when the slave adapter is in BACO mode, the master adapter will use this boost power limit instead of the default power limit to boost the power limit.
		software_shutdown_temp: uint16(),

		reserve: array(uint16(), 8),  //Zero filled field reserved for future use

		power_saving_clock: struct({
			revision: uint8().setValidation(function () {
				return this.value == 0x01 ? true : "Should be 00x1 (Power Saving Clock Table Version 1.00)";
			}),  //Revision = SMU_11_0_7_PP_POWERSAVINGCLOCK_VERSION
			reserve: array(uint8(), 3),  //Zero filled field reserved for future use
			count: uint32(),  //power_saving_clock_count = SMU_11_0_7_PPCLOCK_COUNT
			max: smu_11_0_7_ppclocks,  //PowerSavingClock Mode Clock Maximum array In MHz
			min: smu_11_0_7_ppclocks,  //PowerSavingClock Mode Clock Minimum array In MHz
		}),
		overdrive_table: struct({
			revision: uint8().setValidation(function () {
				return this.value == 0x81 ? true : "Should be 0x81 (OverDrive 8 Table Version 0.2)";
			}),  //Revision = SMU_11_0_7_PP_OVERDRIVE_VERSION
			reserve: array(uint8(), 3),  //Zero filled field reserved for future use
			feature_count: uint32(),  //Total number of supported features
			setting_count: uint32(),  //Total number of supported settings
			cap: struct({
				GfxclkLimits: bool8(),  //GFXCLK Limit feature
				GfxclkCurve: bool8(),  //GFXCLK Curve feature
				UclkLimits: bool8(),  //UCLK Limit feature
				PowerLimit: bool8(),  //Power Limit feature
				FanAcousticLimit: bool8(),  //Fan Acoustic RPM feature
				FanSpeedMin: bool8(),  //Minimum Fan Speed feature
				TemperatureFan: bool8(),  //Fan Target Temperature Limit feature
				TemperatureSystem: bool8(),  //Operating Temperature Limit feature
				MemoryTimingTune: bool8(),  //AC Timing Tuning feature
				FanZeroRpmControl: bool8(),  //Zero RPM feature
				AutoUvEngine: bool8(),  //Auto Under Volt GFXCLK feature
				AutoOcEngine: bool8(),  //Auto Over Clock GFXCLK feature
				AutoOcMemory: bool8(),  //Auto Over Clock MCLK feature
				FanCurve: bool8(),  //Fan Curve feature
				AutoFanAcousticLimit: bool8(),  //Auto Fan Acoustic RPM feature
				PowerMode: bool8(),  //Optimized GPU Power Mode feature,
				Unknown: array(bool8(), 0),
				Unused: array(bool8(), SMU_11_0_7_MAX_ODFEATURE - SMU_11_0_7_ODFEATURE_COUNT),
			}).setUpdate(function (root) {
				const count = Math.max(this.parent.feature_count.value, SMU_11_0_7_ODFEATURE_COUNT);
				this.Unknown.length = count - SMU_11_0_7_ODFEATURE_COUNT;
				this.Unused.length = SMU_11_0_7_MAX_ODFEATURE - count;
			}),  //OD feature support flags
			max: smu_11_0_7_odsettings,  //default maximum settings
			min: smu_11_0_7_odsettings,  //default minimum settings
			pm_setting: struct({
				PowerLimitQuiet: int16(),
				PowerLimitBalance: int16(),
				PowerLimitTurbo: int16(),
				PowerLimitRage: int16(),
				AcousticTempQuiet: int16(),
				AcousticTempBalance: int16(),
				AcousticTempTurbo: int16(),
				AcousticTempRage: int16(),
				Unknown: array(int16(), 8),
				Unused: array(int16(), SMU_11_0_7_MAX_PMSETTING - 16)
			}),  //Optimized power mode feature settings
		}),

		smc_pptable: PPTable  //PPTable_t in smu11_driver_if.h
	}).set({
		byteOrder: "little-endian",
		defaultLockOffset: 0,
	});
	return smu_11_0_7_powerplay_table;
}
